import { forwardRef, EventEmitter, Component, Input, Output, HostListener, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DropdownSettings {
    /**
     * @param {?=} settings
     */
    constructor(settings) {
        this.dropdownHeight = 'auto';
        this.btnWidth = 'auto';
        this.noneSelectedBtnText = 'None selected';
        this.dataIdProperty = 'id';
        this.dataNameProperty = 'name';
        this.deselectAllBtnText = 'Deselect all';
        this.enableFilter = false;
        this.selectAllBtnText = 'Select all';
        this.dropdownClasses = null;
        this.headerText = null;
        this.selectionLimit = null;
        this.showDeselectAllBtn = null;
        this.showSelectAllBtn = null;
        this.btnClasses = null;
        if (!settings)
            return;
        this.noneSelectedBtnText = settings.noneSelectedBtnText || this.noneSelectedBtnText;
        this.dataIdProperty = settings.dataIdProperty || this.dataIdProperty;
        this.dataNameProperty = settings.dataNameProperty || this.dataNameProperty;
        this.deselectAllBtnText = settings.deselectAllBtnText || this.deselectAllBtnText;
        this.enableFilter = settings.enableFilter;
        this.selectAllBtnText = settings.selectAllBtnText || this.selectAllBtnText;
        this.dropdownClasses = settings.dropdownClasses;
        this.headerText = settings.headerText;
        this.selectionLimit = settings.selectionLimit;
        this.showDeselectAllBtn = settings.showDeselectAllBtn;
        this.showSelectAllBtn = settings.showSelectAllBtn;
        this.btnWidth = settings.btnWidth || this.btnWidth;
        this.dropdownHeight = settings.dropdownHeight || this.dropdownHeight;
        this.btnClasses = settings.btnClasses;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DROPDOWN_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef((/**
     * @return {?}
     */
    () => NgxBootstrapMultiselectDropdownComponent)),
    multi: true
};
class NgxBootstrapMultiselectDropdownComponent {
    constructor() {
        this.items = [];
        this.onDataSelect = new EventEmitter();
        this.onSelectAllData = new EventEmitter();
        this.onDeselectAllData = new EventEmitter();
        this.filteredItems = [];
        this.filterValue = '';
        this.isVisible = false;
        this.selectedItems = []; // Used to keep track if items currently selected. Dropdown item ids are stored
        // ControlValueAccessor methods
        this.onChange = (/**
         * @param {?} selectedObject
         * @return {?}
         */
        (selectedObject) => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
    }
    // Used to keep track if items currently selected. Dropdown item ids are stored
    // Close dropdown when clicking outside component
    /**
     * @return {?}
     */
    clickOutsideComponent() {
        if (this.isVisible)
            this.isVisible = false;
    }
    // Prevent event reaching clickOutsideComponent 
    /**
     * @param {?} event
     * @return {?}
     */
    handleComponentClick(event) {
        event.stopPropagation();
    }
    // Return selected items by filtering items array based on values in selectedItems array
    /**
     * @return {?}
     */
    getSelectedItems() {
        this.setSelectedText();
        return this.items
            .filter((/**
         * @param {?} _
         * @return {?}
         */
        _ => this.selectedItems.findIndex((/**
         * @param {?} x
         * @return {?}
         */
        x => x === _[this.innerSettings.dataIdProperty])) > -1)) // Return only items with id values in selectedItems
            .slice();
    }
    // Set text when selecting item from dropdown
    /**
     * @return {?}
     */
    setSelectedText() {
        this.selectedText = this.selectedItems.length
            ? `${this.selectedItems.length} item${this.selectedItems.length > 1 ? 's' : ''} selected`
            : this.innerSettings.noneSelectedBtnText;
    }
    // Toggle dropdown visibility
    /**
     * @return {?}
     */
    showDropdown() {
        this.isVisible = this.isVisible ? false : true;
    }
    // Filter items based on item name property value
    /**
     * @param {?} value
     * @return {?}
     */
    onFilterSearch(value) {
        this.filterValue = value; // Save filter value so it appears when toggling dropdown
        this.filteredItems = this.items.filter((/**
         * @param {?} _
         * @return {?}
         */
        _ => _[this.innerSettings.dataNameProperty] && _[this.innerSettings.dataNameProperty].toLowerCase().startsWith(value)));
    }
    // Set selected dropdown item as active. Activated on dropdown item click
    /**
     * @param {?} selectedObject
     * @return {?}
     */
    onSelect(selectedObject) {
        if (this.disabled)
            return;
        this.onTouched();
        this.writeValue(selectedObject);
        this.onSelectAllData.emit(selectedObject);
    }
    // Set all dropdown items as active. Activated on select all button item click
    /**
     * @return {?}
     */
    onSelectAll() {
        if (this.disabled)
            return;
        this.onTouched();
        this.selectedItems = this.items.map((/**
         * @param {?} _
         * @return {?}
         */
        _ => _[this.innerSettings.dataIdProperty]));
        this.writeValue([]);
        this.onDataSelect.emit();
    }
    // Remove active from all dropdown items. Activated on deselect all button item click
    /**
     * @return {?}
     */
    onDeselectAll() {
        if (this.disabled)
            return;
        this.onTouched();
        this.selectedItems = [];
        this.writeValue([]);
        this.onDeselectAllData.emit();
    }
    // Check if number of selected items is equal or greater than limit
    /**
     * @return {?}
     */
    isSelectionLimitReached() {
        return this.innerSettings.selectionLimit && this.innerSettings.selectionLimit <= this.selectedItems.length;
    }
    // Check if drowdown item is active
    /**
     * @param {?} item
     * @return {?}
     */
    isActive(item) {
        return this.selectedItems.findIndex((/**
         * @param {?} x
         * @return {?}
         */
        x => x === item[this.innerSettings.dataIdProperty])) > -1;
    }
    // Check if input values exist in selectedItems array. If item exists remove from array, else add
    /**
     * @param {?} selectedObject
     * @return {?}
     */
    writeValue(selectedObject) {
        if (selectedObject) {
            /** @type {?} */
            const tempArray = Array.isArray(selectedObject) ? (/** @type {?} */ (selectedObject)) : [selectedObject];
            if (tempArray.length === 0) {
                this.selectedItems = [];
            }
            else {
                this.items = this.items.map((/**
                 * @param {?} _
                 * @return {?}
                 */
                _ => {
                    /** @type {?} */
                    var index = tempArray.findIndex((/**
                     * @param {?} x
                     * @return {?}
                     */
                    x => _[this.innerSettings.dataIdProperty] === x[this.innerSettings.dataIdProperty]));
                    if (index > -1) {
                        /** @type {?} */
                        const index = this.selectedItems.findIndex((/**
                         * @param {?} x
                         * @return {?}
                         */
                        x => x === _[this.innerSettings.dataIdProperty]));
                        if (index > -1) {
                            this.selectedItems.splice(index, 1);
                        }
                        else {
                            if (!this.isSelectionLimitReached()) {
                                this.selectedItems.push(_[this.innerSettings.dataIdProperty]);
                            }
                        }
                    }
                    return _;
                })).slice();
            }
        }
        this.onChange(this.getSelectedItems());
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.innerSettings = new DropdownSettings(this.settings); // Set initial setting values
        this.filteredItems = this.items; // Set initial filtered values
        this.setSelectedText(); // Set initial button text
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
}
NgxBootstrapMultiselectDropdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-bootstrap-multiselect',
                template: `
  <div class="dropdown" (click)="handleComponentClick($event)" >
    <button style="text-align: right"
      [disabled]="disabled"
      type="button" 
      (click)="showDropdown()"
      [ngClass]="innerSettings.btnClasses"
      [style.width]="innerSettings.btnWidth">
      <span style="float: left">{{selectedText}}</span>
      
    </button>
    <div *ngIf="isVisible" class="dropdown-menu pointer" [ngClass]="innerSettings.dropdownClasses" aria-labelledby="triggerId" style="display: inline-block">
      <div class="dropdown-header" *ngIf="innerSettings.headerText">{{innerSettings.headerText}}</div>
      <div *ngIf="innerSettings.showSelectAllBtn && !innerSettings.selectionLimit" class="dropdown-item" (click)="onSelectAll()">{{innerSettings.selectAllBtnText}}</div>
      <div *ngIf="innerSettings.showDeselectAllBtn" class="dropdown-item" (click)="onDeselectAll()">{{innerSettings.deselectAllBtnText}}</div>
      <div *ngIf="innerSettings.enableFilter" class="p-2"><input autocomplete="off" list="autocompleteOff" type="text" placeholder="Filter values" [value]="filterValue" (keyup)="onFilterSearch($event?.target?.value)" class="form-control form-control-sm" /></div>
      <div class="dropdown-divider" *ngIf="innerSettings.showSelectAllBtn || innerSettings.showDeselectAllBtn || innerSettings.enableFilter"></div>
      <div [style.height]="innerSettings.dropdownHeight" style="overflow: auto" >
        <div *ngFor="let item of filteredItems; let i=index" (click)="onSelect(item)" class="dropdown-item" [ngClass]="{'active': isActive(item), 'disabled': disabled }">
          {{item[innerSettings.dataNameProperty]}}            
        </div>    
      </div>  
    </div>
  </div>
  `,
                providers: [DROPDOWN_CONTROL_VALUE_ACCESSOR],
                styles: ['.pointer > .dropdown-item { curson: pointer; }']
            }] }
];
NgxBootstrapMultiselectDropdownComponent.propDecorators = {
    disabled: [{ type: Input }],
    items: [{ type: Input }],
    settings: [{ type: Input }],
    onDataSelect: [{ type: Output }],
    onSelectAllData: [{ type: Output }],
    onDeselectAllData: [{ type: Output }],
    clickOutsideComponent: [{ type: HostListener, args: ['document:click',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBootstrapMultiselectDropdownModule {
}
NgxBootstrapMultiselectDropdownModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxBootstrapMultiselectDropdownComponent],
                imports: [
                    CommonModule,
                    FormsModule
                ],
                exports: [NgxBootstrapMultiselectDropdownComponent]
            },] }
];

export { DROPDOWN_CONTROL_VALUE_ACCESSOR, NgxBootstrapMultiselectDropdownComponent, NgxBootstrapMultiselectDropdownModule };
//# sourceMappingURL=ngx-bootstrap-multiselect-dropdown.js.map
